#! /bin/sh
SHELL=/bin/sh
export SHELL
###############################################################################
#
# FILE        : findlib.sh
#
# INPUT       : $INFORMIXDIR
#
# OUTPUT      : environment files for Bourne shell and C shell.
#
# DESCRIPTION : Initialize FGLLIBSQL and FGLLIBSYS in file envf4gl.
#               Be sure to have $INFORMIXDIR variable already set
#               before running this shell.
#               Then just execute ./findlib.sh
#
#               NOTE:
#                   The following file is used by findlib.sh:
#                       $FGLDIR/lib/testbase.42m
#                   All other required files are self extracted from
#                   the shell script.
# DATE        : 20/11/96
# UPDATE      : 03/09/97. "All-in-one" shell script
#             : 26/09/97. Small changes / translation
# Modif E.S   : 22/01/98. function detect_operating_system() divided
#                         in two functions: detect_operating_system and
#                         detect_operating_flag.
#                         New hidden flag -sys which specify the OS name.
#                         The next parameter must be the output of a uname.
#                         In this case the operating system name
#                         is set in a global variable "system_name".
#                         Other new hidden flags specify if findlib.sh are 
#                         called by install.sh. In this case a global variable
#                         "from_install" is set to y, otherwise is set to n
#                         Last hidden flag: -nocls. If set, the screen is not
#                         cleared at the begining of the script.
#             : 16/02/98. Modification of the function create_envf4gl.
#                         Add the FGLDIR/bin and INFORMIXDIR/bin in the path.
# Modif E.S   : 20/02/98. Only for Linux Os, findlib.sh tests if the compiled
#                         file is from COFF or ELF type. For ELF type, the shell
#                         stop and ask user to use the GNU C Compiler which
#                         is available on the CD-ROM. 
# Modif E.S   : 23/04/98  On DG-UX the flag -r for the following statement
#                         while read $readflag line, must not be used.
# Modif E.S   : 24/04/98  On sgi with Dynamic Server the variable
#                         numpf is not initialized
# Modif E.S   : 30/04/98  This modification replace the modification from the
#                         24/04/98. Now DG-UX is fully integrated in the
#                         findlib.sh.
# Modif JPK   : 05/05/98  Creation of environment file envf4gl.csh for C Shell
# Modif E.S   : 29/05/98  Adding new messages.
# Modif SF    : 03/06/98  Normalization of Usage display.
# Modif JPK   : 09/07/98  Accept Informix Linux native version, but give a
#                         warning for COFF compliance.
# Modif E.S   : 24/07/98  Correction of the INFORMIXDIR0 bug. This bug is 
#                         caused by the result of the command "expr". In 
#                         fact if the 'expr' statement is not able to find 
#                         the required value it returns an error
#                         code but not in the shell status like the order 
#                         statement like 'df' but in the stdout.
# Modif E.S   : 04/08/98  Now, it's possible to use a shell compiler instead
#    Version  : 2.04.1a   of the C Compiler. This new flag is '-sh' after that
#                         the name of the shell compiler must be specified.
# Modif E.S   : 16/09/98  After several test the flag 'readflag' has been 
#			  supress from findlib because it was obsolete.
#			  Now the NCR MP-RAS is fully supported.
# Modif E.S   : 30/09/98  There was bug with the flag '-sh'. When findlib.sh
#    Version  : 2.04.1c   founded a shell compiler, the function which found
#                         the informix version and the libraries needed to 
#                         link the P code runner or compile in C code 
#                         were not exuted. In this case the
#                         variables FGLCC, FGLDBS, FGLLIBSYS and FGLLIBSQL
#                         were set to NULL.
#                         Now, even a shell compiler is found and the 
#                         compilation is successfull, findlib.sh don't stop. 
#                         But when the files envf4gl, envf4gl.csh is created the
#                         variables FGLLIBSQL and FGLLIBSYS are put with
#                         # ( for comment ).
# Modif E.S   : 05/10/98  Adding the support of the newest Informix database
#    Version  : 2.04.2b   7.30 and Informix generic interface.
# 
# Modif E.S   : 02/11/98  Adding a new functionality which create in the file
#                         envf4gl the envinronment variable LD_LIBRARY_PATH
#
# Modif E.S   : 07/12/98  Correction of the LD_LIBRARY_PATH management.  
#    Version  : 2.05.1a   Now, the LD_LIBRARY_PATH variable is created as
#                         follow:
#                           LD_LIBRARY=$INFORMIXDIR/lib:$LD_LIBRARY_PATH
#                         Several customer, ask us to uncomment "LIBRARY" line
#                         if the FGLSHELL variable exist.
#                         The algorithm for finding the Informix product
#                         interface version has changed.
#                         in C shell, setenv SHELL /bin/sh had replaced by
#                                     setenv SHELL /bin/csh
#                         Accept ESQL version 9.xx
# Modif E.S   : 08/12/98  On H.P, we need to test if LD_LIBRARY_PATH is valuate
#    Version  : 2.05.1b
###############################################################################

######################### GLOBALS ###########################################
tmpdir="/tmp/f4tmp$$"
#tmpdir="`pwd`/f4tmp"
source_dir=`pwd`
progname=`basename $0`
have_env=0
numpf=0
system_name=""
from_install=""
expr_version=""
share_lib_path=""
SHELL_COMMENT=""
######################### TRAP ##############################################
stop_program() {
  echo "$progname aborted. Exiting."
  if [ $have_env = 1 ];then
    echo "But you have a 'envf4gl' or 'envf4gl.csh' file generated."
    echo "Minimum number of undefined link functions is: $minlfct"
    echo "Run next commands to see exactly which one:"
    echo ""
    echo "In Bourne Shell:"
    echo ". ./envf4gl"
    if [ -n "$opt" ]
     then
      opt=" -add $opt"
    fi
    echo "fglmkrun$opt"
    echo ""
    echo "or in C Shell:"
    echo "source ./envf4gl.csh"
    echo "fglmkrun$opt"
  fi
  exitp 1
}
trap 'stop_program' 1 2 3 15

#############################################################################
# Search type of echo 
##############################################################################
a=`echo -n "ceci"`
if [ $a = "-n ceci" ]
then
  begech=""
  endech="\c"
else
  begech="-n"
  endech=""
fi
echo_nonl(){
echo $begech "$*$endech"
}

#############################################################################
# strlen
##############################################################################
strlen(){
a=`echo_nonl $1|wc -c`
echo $a
}

#############################################################################
# Formatted line
##############################################################################
points=".........................................................."
echo_format() {
 str="$1$points"
 line=`expr "$str" : "\(..................................................\).*"`
 echo_nonl "$line "
}

exitp() {
#echo " rm -rf $tmpdir"
  cd $source_dir
  rm -rf $tmpdir
  exit $1
}

#############################################################################
# Usage
##############################################################################
usage() {
echo "Search libraries to link the runner"
echo "Usage: $progname [-h] [-V] [-l] [-max] [-o runner] [-sh name] [-tl opt]"
echo "  -l       : Search libraries and create 'envf4gl' and 'envf4gl.csh'."
echo "             This is the default behavior."
echo "  -o runner: Attempting to build the runner (if possible)."
echo "  -max     : Attempting to build the runner even without Informix's"
echo "             script. Searches all systems libraries."
echo "             This can take a long time."
echo "  -V       : Display version information."
echo "  -h       : Display this help."
echo "  -sh name : Specify which Informix script you want to search the links."
echo "             Default list is 'esql c4gl cfglgo'."
echo "  -tl opt  : Only available with Informix version 7.x."
echo "             Where 'opt' can be one of the following :"
echo "                 static : Use static libraries (default)."
echo "                 shared : Use shared libraries."
exit 0
}

#############################################################################
# Self extract file
##############################################################################
self_extract() {
if [ ! -r "$1" ];then
  echo "Cannot read file '$1'"
  exitp 1 
fi
if [ -z "$2" ];then
  echo "You must specify which file to extract."
  exitp 1 
fi
grep "@$2@" $1|sed "s/@$2@//" >$2
a=`cat $2|wc -l`
if [ $a = 0 ];then
  echo "Cannot extract file '$2' from '$1'"
  exitp 1
fi
}

#############################################################################
# void std_get_undef_fct(void)
#
# INPUT       : none
# OUTPUT      : undef
#
# DESCRIPTION : Search undef function
# DATE        : 04/09/97
##############################################################################

std_get_undef_fct() {
  filename="$1"
  lfct=""
  knownadd=""
  while read line
  do
    if [ -n "$line" ];then
      found=0
      set $line
      if [ -n "$badlink1" ];then
        no=0
        for b1 in $badlink1
        do
          b1=`echo $b1|sed "s/@@@/$filename/g"`
          no=`expr $no + 1`
          if [ "$1" = "$b1" ];then
            found=1
            break
          fi
        done
      fi
      if [ $found = 1 -a -z "$2" ];then
        continue
      fi
      if [ $found = 1 ];then
        found=0
        if [ -n "$badlink2" ];then
          i=0
          for b2 in $badlink2
          do
            i=`expr $i + 1`
            if [ $no != $i ];then
              continue
            fi
            b2=`echo $b2|sed "s/@@@/$filename/g"`
            #echo "par2 $2 b2 $b2"
            if [ "$2" = "$b2" ];then
              continue 2
            fi
          done
        fi
      fi
      p=0
      for i in $line
      do
        p=`expr $p + 1`
        if [ $p = $numpf ];then
          if [ -n "$apt" ];then
           if [ "$expr_version" = 1.16 ]
            then
             f=`expr "$i" : ".\(..*\)"`
            else
             f=`expr "$i" : "^.\(..*\)"`
           fi 
            fct=$f
          else
            fct=$i
          fi
          lfct="$lfct $fct"
          if [ $max = 1 ];then
            verify_known_fct $fct <unref.dat
          fi
          nblfct=`expr $nblfct + 1`
        fi
      done
    fi
  done
}

get_undef_fct() {
  nblfct=0
  if [ $foundbl = 0 ];then
    return
  fi
  if [ ! -r "$2" ];then
    return
  fi
  grep -v -i "warning" $2 >$tmpdir/undef
  std_get_undef_fct $1 <$tmpdir/undef
}

#############################################################################
# void search_badlink(void)
#
# INPUT       : none
# OUTPUT      : badlink
#
# DESCRIPTION : Search line of bad link
# DATE        : 04/09/97
##############################################################################

std_search_badlink() {
  apt=""
  while read line
  do
    if [ -n "$line" ];then
      found=0
      set $line
      p=0
      for i in $*
      do
        p=`expr $p + 1`
        if [ $i = "extfunct3" -o $i = "extfunct4" -o $i = "extfunct5" ];then
          numpf=$p
          found=1
          break
        fi
        if [ $i = ".extfunct3" -o $i = ".extfunct4" -o $i = ".extfunct5" ];then
          numpf=$p
          apt="."
          found=1
          break
        fi
      done
      if [ $found = 0 ];then
        par=`echo $1|sed 's/tenm/@@@/g'`
        badlink1="$badlink1 $par"
        par=`echo $2|sed 's/tenm/@@@/g'|sed 's/ *//g'`
        if [ -z "$par" ];then
          badlink2="$badlink2 \"\""
        else
          badlink2="$badlink2 $par"
        fi
      fi
    fi
  done
  foundbl=1
}

search_badlink() {
  if [ ! -r "$1" ];then
    return
  fi
  grep -v -i "warning" $1 >$tmpdir/undef
  std_search_badlink <$tmpdir/undef
}

#############################################################################
# void verify_compiler(void)
#
# INPUT       : none
# OUTPUT      : ANSICFLAGS
#
# DESCRIPTION : Verify C compiler, attempting to find ANSI C compilation option
# DATE        : 04/09/97
##############################################################################

verify_compiler() {

cd $tmpdir

echo_format "Searching C compiler"
lcc="$FGLCC $CC cc gcc"
foundcc=0
for cc in $lcc
do
  type $cc >/dev/null 2>/dev/null
  if [ $? = 0 ];then
    foundcc=1
    FCC=$cc
    break
  fi
done
if [ $foundcc = 1 ];then
  echo "found \"$FCC\""
  if [ "$unix" = Linux ]
   then
    echo_format "Searching C compiler type"
    flag_elf="n"
    $FCC -c tenm.c 2>/dev/null
    file tenm.o 1>Ctype
    while read line
     do
      for i in $line 
       do
        if [ "$i" = ELF ]
         then
          flag_elf="y"
        fi
        if [ "$flag_elf" = y ]
         then
          break
        fi
      done
    done<Ctype
    if [ "$flag_elf" = y ]
     then
      echo "Elf Compliant"
      echo ""
      echo "Warning! This is only available with an Informix Linux native version."
      echo "To Compile the runner with SCO libraries,"
      echo "you must have a COFF compliant C compiler."
      echo "(You found the COFF GNU C Compiler for Linux on the CD-ROM)"
      echo ""
      echo "no" 1>$FGLDIR/bin/no
      #exitp 1
     else
      echo "Coff Compliant"
    fi
  fi
else
  echo "Not found"
  echo "Check your PATH and/or your C compiler name then run this program again."
  echo "Exiting program."
  exitp 1
fi

echo_format "Verify ANSI C compatibility"
ANSICFLAGS=${ANSICFLAGS=""}
foundcc=0
for ansif in "$ANSICFLAGS" "-Aa"
do
  $FCC $ansif -c tenm.c >/dev/null 2>/dev/null
  if [ $? = 0 ];then
    if [ -n "$ansif" ];then 
      ANSICFLAGS=$ansif
      export ANSICFLAGS
    fi
    foundcc=1
    break
  fi
done
if [ $foundcc = 0 ];then
  echo "No"
  echo " WARNING! If you want to include C functions,"
  echo "          you must have a ANSI C compatible compiler."
  echo " Please, check your compiler documentation to find the required options"
  echo "         then set the environment variable ANSICFLAGS and run this program again"
  cp $tmpdir/natenm.c $tmpdir/tenm.c 2>/dev/null
  if [ $? != 0 ];then
    echo "Cannot copy file $tmpdir/natenm.c."
    echo "Please, check the access rights and run this program again"
    exitp 1
  fi
else
  echo "Ok"
fi
foundbl=0
if [ $max = 0 ];then
  return
fi
echo_format "Check linker to find undefined functions"
badlink1=""
badlink2=""
$FCC $ANSICFLAGS -o tenm tenm.c >$tmpdir/badlink 2>&1
#cat $tmpdir/badlink
search_badlink  $tmpdir/badlink
#echo "bl1 $badlink1 bl2 $badlink2"
get_undef_fct tenm  $tmpdir/badlink
foundbadlink=0
if [ -n "$lfct" ];then
  nb=0
  for i in $lfct
  do
    if [ "$i" = "extfunct3" -o "$i" = "extfunct4" -o "$i" = "extfunct5" ];then
      nb=`expr $nb + 1`
    fi
  done
  if [ $nb = 3 ];then
    foundbadlink=1
  fi
fi
if [ $foundbadlink = 0 ];then
  echo "cannot found them"
else
  echo "Ok"
fi
depend=0
echo_format "Search type of link order"
$FCC $ANSICFLAGS -c teli.c >$tmpdir/badlink 2>&1
$FCC $ANSICFLAGS -c teli1.c >$tmpdir/badlink 2>&1
$FCC $ANSICFLAGS -c teli2.c >$tmpdir/badlink 2>&1
ar rv libt1.a teli1.o 2>/dev/null 1>&2
if [ $? != 0 ];then
  echo "No"
  echo "Cannot create a library with \"ar\" command"
  return
fi
ar rv libt2.a teli2.o 2>/dev/null 1>&2
if [ $? != 0 ];then
  echo "No"
  echo "Cannot create a library with \"ar\" command"
  return
fi
$FCC $ANSICFLAGS -o teli teli.o libt1.a libt2.a >$tmpdir/badlink 2>&1
if [ $? != 0 ];then
  get_undef_fct teli  $tmpdir/badlink
  if [ -n "$lfct" -a "`echo \"$lfct\"|sed 's/ *//g'`" = "es1" ];then
    echo "Depend"
    depend=1
  else
    echo "No"
    echo "WARNING ! Unknown error: $lfct"
  fi
else
  echo "Ok. Not dependent"
fi
}

#############################################################################
# void search_nmline(void)
#
# INPUT       : none
# OUTPUT      : nmcmd
#
# DESCRIPTION : Execute and check the nm command to extract function names
# DATE        : 04/09/97
##############################################################################

search_nmline() {
  foundnm=0
  while read line
  do
    l=`echo "$line"|sed 's/@/\\\\/g'`
    #echo "eval \"nm tenm.o $l|sort\""
    a=`eval "nm tenm.o $l|sort" 2>/dev/null`
    #echo "res: $a"
    if [ -n "$a" ];then
      set $a
      if [ $# = 3 -a "$1" = "funct1" -a "$2" = "funct2" -a "$3" = "main" ];then
        nmcmd="$l"
        a=`eval "nm tenm.o $nmcmd|sort" 2>/dev/null`
        set $a
        if [ $# = 3 -a "$1" = "funct1" -a "$2" = "funct2" -a "$3" = "main" ];then
          foundnm=1
          break
        else
          nmcmd=""
        fi
      fi
    fi
  done
}

#############################################################################
# void search_nmcmd(void)
#
# INPUT       : none
# OUTPUT      : nmcmd
#
# DESCRIPTION : Search nm command to extract function name of object file
# DATE        : 04/09/97
##############################################################################

search_nmcmd() {

if [ $foundcc = 0 ];then
  return
fi
echo_format "Search \"nm\" command to extract functions names"

type nm >/dev/null 2>/dev/null
if [ $? != 0 ];then
  echo "No"
  echo " Cannot found command \"nm\""
  return
fi
cd $tmpdir
$FCC $ANSICFLAGS -c tenm.c >/dev/null 2>/dev/null
if [ $? != 0 ];then
  echo "No"
  echo " Cannot compile file tenm.c"
  return
fi
if [ ! -r tenm.o ];then
  echo "No"
  echo " Cannot found file tenm.o"
  return
fi
cd $tmpdir
search_nmline < $tmpdir/nm.cmd
if [ $foundnm = 1 ];then
  echo "Ok"
else
  echo "No"
fi
}

#############################################################################
# void detect_operating_system(void)
#
# INPUT       : none
# OUTPUT      : os_name       = Operating System name
#
# DESCRIPTION : Attempt to detect Operating System
#               This is a public function
#
# DATE        : 28/05/96
##############################################################################

detect_operating_system() {

 os=""
 tmp_4js=""
 i=""
 flag=""
 unix=""

 if [ ! -z "$system_name" ]
  then
   unix=$system_name
  else
   unix=`uname`
 fi
 flag=yes
 i=0

 for i in "$unix"; do
  case $i in
   Linux)         os=1;;     # Linux
   SCO)           os=2;;     # SCO Unix
   SunOS)         os=3;;     # Sun
   AIX)           os=4;;     # IBM AIX
   UNIX_SV)       os=5;;     # SCO or Novell UnixWare
   OSF1)          os=6;;     # DEC (Digital Equipment Corporation)
   HP-UX)         os=7;;     # Hewlett-Packard
   SINIX-N)       os=8;;     # Siemens
   SCO_SV)        os=9;;     # SCO Open Server
   IRIX)          os=10;;    # Irix System
   dgux)          os=11;;    # DG-UX system
   MPRAS)         os=12;;    # MP-RAS 3.x
   "CYGWIN32/NT") 
		  echo "Microsoft Windows NT not supported"
                  echo
                  echo 'Installation aborted. Exiting configure program.'
                  exitp 1
		  ;;
   *)             
     flag=no;;
  esac
 done

 if [ $flag = "no" ]; then
  loop=y;
  until [ "$loop" = n ]; do
   echo 'Operating System unknown'
   echo 'Please, enter your current Operating System: '
   echo
   echo ' 0. Operating System is not mentioned below'
   echo ' 1. Linux'
   echo ' 2. SCO version 3.2.x or earlier version'
   echo ' 3. Sun Solaris 2.x'
   echo ' 4. IBM AIX'
   echo ' 5. SCO/Novell UnixWare or Unix System V Rel.4 Intel based compatible'
   echo ' 6. DEC Digital UNIX'
   echo ' 7. Hewlett-Packard HP-UX'
   echo ' 8. Siemens SINIX or Mips System 5.4 compatible'
   echo ' 9. SCO Open server'
   echo '10. IRIX System'
   echo '11. DG-UX System'
   echo '12. NCR MP-RAS Unix 3.x'
   echo 
   echo_nonl 'Which Operating System are you using [0-12] ? '
   read os
   case $os in
    [0-9]|10|11|12) loop=n;;
    *) echo
       echo 'Wrong option.'
       echo;;
   esac
  done
  echo 
 fi
 detect_operating_flags $os
}

#############################################################################
# void detect_operating_flags(char)
#
# INPUT       : os
# OUTPUT      : os_name       = Operating System name
#
# DESCRIPTION : Attempt to set flags for the current Operating System 
#               This is a public function
#
# DATE        : 28/05/96
##############################################################################

detect_operating_flags() {
 os="$1"
 tryopt=""
 tryopt2=""
 case $os in
  0) echo
     echo 'Please, install a compatible Operating System! : EXIT'
     echo
     echo 'Installation aborted. Exiting configure program.'
     exitp 1;;
  1) os_name="LINUX"
     sys4js="-lm"
     sys4js2="-lm -lsocket"
     echo '--portability |grep " T "|cut -f1 -d" "' >$tmpdir/nm.cmd
     ;;
  2) os_name="SCO"
     tryopt2="-b elf"
     sys4js="-lm -lsocket"
     sys4js2="-lnsl_s -lcrypt_i -lsocket -lm -lsuds -lx -lmalloc"
     echo '|grep "extern"|grep "@.text"|cut -f1 -d" "' >$tmpdir/nm.cmd
     ;;
  3) os_name="SUN"
     sys4js="-lm -lsocket -lnsl"
     echo '|grep "FUNC"|grep "GLOB"|grep -v "UNDEF"|sed "s/^..*@|@([^@|]*@)$/@1/g"' >$tmpdir/nm.cmd
     ;;
  4) os_name="AIX"
     sys4js="-lm"
     echo '|grep " T "|sed "s/^.@([^( ]*@).*$/@1/"' >$tmpdir/nm.cmd
     echo '|grep "extern"|grep "@.text"|sed "s/@^.@([^( ]*@).*$/@1/g"' >>$tmpdir/nm.cmd
     ;;
  5) os_name="UNIXWARE"
     sys4js="-lm -lsocket -lnsl"
     sys4js2="-lm -lsocket -lnsl -lx"
     echo '|grep "FUNC"|grep "GLOB"|grep -v "UNDEF"|sed "s/^..*@|@([^@|]*@)$/@1/g"' >$tmpdir/nm.cmd
     ;;
  6) os_name="DEC"
     sys4js="-lm"
     echo '|grep " T "|cut -f1 -d" "' >$tmpdir/nm.cmd
     ;;
  7) os_name="HP"
     sys4js="-lm"
     echo '|grep "extern"|grep "entry"|cut -f1 -d" "' >$tmpdir/nm.cmd
     ;;
  8) os_name="SINIX"
     sys4js="-lm -lsocket -lnsl"
     echo '|grep "FUNC"|grep "GLOB"|grep -v "UNDEF"|sed "s/^..*@|@([^@|]*@)$/@1/g"' >$tmpdir/nm.cmd
     ;;
  9) os_name="SCO-O.S"
     tryopt="-b elf"
     sys4js="-lnsl_s -lcrypt_i -lsocket -lm -lsuds -lx -lmalloc"
     sys4js2="-lm -lsocket"
     echo '|grep "extern"|grep "@.text"|cut -f1 -d" "' >$tmpdir/nm.cmd
     ;;
 10) os_name="IRIX"
     sys4js="-lm"
     echo '-B|grep " T "|grep weak|cut -f3 -d" "' >$tmpdir/nm.cmd
     echo '-B|grep " T "|cut -f3 -d" "' >>$tmpdir/nm.cmd
     ;;
 11) os_name="DG-UX"
     sys4js="-lm -lsocket -lnsl"
     sys4js2="-lm -lsocket -lnsl -lx"
     echo '|grep "FUNC"|grep "GLOB"|grep -v "UNDEF"|sed "s/^..*@|@([^@|]*@)$/@1/g"' >$tmpdir/nm.cmd
     ;;
  12) os_name="MP-RAS 3.x"
     sys4js="-lm -lsocket -lnsl"
     sys4js2="-lm -lsocket -lnsl -lx"
     echo '|grep "FUNC"|grep "GLOB"|grep -v "UNDEF"|sed "s/^..*@|@([^@|]*@)$/@1/g"' >$tmpdir/nm.cmd
     ;;
  *) echo 'Error: wrong choice : EXIT'
     echo 'Installation aborted. Exiting configure program.'
     exitp 1;;
 esac
 echo '|grep "extern"|grep "@.text"|cut -f1 -d" "' >>$tmpdir/nm.cmd
 echo '|grep "FUNC"|grep "GLOB"|grep -v "UNDEF"|sed "s/^..*@|@([^@|]*@)$/@1/g"' >>$tmpdir/nm.cmd
 echo '|grep " T "|sed "s/^.@([^( ]*@).*$/@1/"' >>$tmpdir/nm.cmd
 echo '|grep "extern"|grep "@.text"|sed "s/@^.@([^( ]*@).*$/@1/g"' >>$tmpdir/nm.cmd
 echo '|grep " T "|cut -f1 -d" "' >>$tmpdir/nm.cmd
 echo '|grep "extern"|grep "entry"|cut -f1 -d" "' >>$tmpdir/nm.cmd
 echo '--portability |grep " T "|cut -f1 -d" "' >>$tmpdir/nm.cmd
 echo '-B|grep " T "|grep weak|cut -f3 -d" "' >>$tmpdir/nm.cmd
 echo '-B|grep " T "|cut -f3 -d" "' >>$tmpdir/nm.cmd

}

search_linkline() {
  while read line
  do
    a=`echo "$line"|grep "^$cc "`
    if [ -z "$a" ];then
      a=`echo "$line"|grep " $cc "`
    fi
    if [ -n "$a" ];then
      linkline=$line
      foundlink=1
      break
    fi
  done

}

#############################################################################
# get_compile_line()
#
# INPUT       : $1 = Informix command where I have to extract
#
#
# DESCRIPTION : Run a compile or a link of Informix product
#               Search line that contains the compile
# DATE        : 03/09/97
##############################################################################

get_compile_line() {

cd $tmpdir
foundlink=0
cp $informixdir/bin/$infcc $tmpdir 2>/dev/null
if [ $? != 0 ];then
  echo "Cannot copy file $informixdir/bin/$infcc."
  echo "Please, check your access rights and run this program again"
  exitp 1
fi
echo_format "Search Informix libraries type"
flib=$typlib
if [ $typlib = "-static" -a $infcc != "cfglgo" ];then
  a=`$infcc 2>&1|grep "static"|wc -l 2>/dev/null`
  if [ $a = 0 ];then
    flib=""
  fi
fi
for opt in $flib ""
do
  #echo "sh -x $infcc $opt -o linkix $file >/dev/null  2>tracex.sh"
  sh -x $infcc $opt -o linkix $file >/dev/null  2>tracex.sh
  T=$?
  if [ $? = 0 ];then
    case "$opt" in 
	-static|static) echo "static"
		;;
	-shared|shared) echo "shared"
		;;
	*) echo "No specific type"
		;;
    esac
    break
  fi
done
if [ $T != 0  ];then
  echo "Warning! Cannot link an Informix program with \"$infcc\""
  echo "The following error occured: "

  $infcc $opt -o linkix $file  >error1 2>error2
  echo "------------------------------------------"
  if [ `cat error2|wc -l` = 0 ];
  then
    cat error1
  else
    cat error2
  fi
  echo "------------------------------------------"
  echo "Please, check your \"$infcc\" shell script:"
  echo "        - if the PATH to access the development tools ( C compiler, linker...)"
  echo "          is correct."
  echo "        - if the definition of CC is not hard coded into the shell on a wrong"
  echo "          compiler product. In this case, set it to the correct value and"
  echo "          also check the C compiler options line."
  foundscript=0
  return
fi
echo_format "Search Informix link line"
lcc="$FGLCC $CC cc gcc"
for cc in $lcc
do
  search_linkline < tracex.sh
  if [ -n "$linkline" ];then 
    break
  fi
done
if [ $foundlink = 1 ];then
  echo "Ok"
else
  echo "No"
fi
}

#############################################################################
# verify_known_fct()
#
# DESCRIPTION : Verify known unrerenfed
#
# DATE        : 03/09/97
##############################################################################
verify_known_fct() {

if [ $nblfct -gt 10 ];then
  return
fi
fct="$1"
while read linev
do
  for i in $linev
  do
    if [ $i = $fct ];then
      set $linev
      shift
      a=`eval "echo $*"`
      knownadd="$knownadd $a"
      break
    fi 
    break
  done
done
}

#############################################################################
# extract_informix_libs()
#
# INPUT       : $1 = Line that contains link
#
#
# DESCRIPTION : Extract informix libs from line
#               This is a public function.
# DATE        : 03/09/97
##############################################################################

extract_informix_libs() {

 #echo "$linkline"
 echo_format "Search libraries in link line"
 libs=""
 optdirlibs=""
 dirlibs=""
 if [ $infcc != "esql" ];then
   libsix4gl="ncolor fmain fglgo nforms 4gl 4io nmenu rdsterm fe"
   libstest="x bsd curses termlib termcap terminfo "
 else
   libsix4gl=""
   libstest="x bsd"
 fi
 
 
 next="no"
 for p in $linkline
 do
  if [ $p = "+" ];then
    continue
  fi
  if [ "$expr_version" = 1.16 ]
   then
    opt=`expr "$p" : "\(..\).*"`
   else
    opt=`expr "$p" : "^\(..\).*"`
  fi
  case $opt in
	-s)
           ;;
        -I) next="I"
	    ;;
        -o) next="link"
	    ;;
        -L) if [ $opt = $p ];then
              next="L"
            else
              optdirlibs="$optdirlibs $p"
              if [ "$expr_version" = 1.16 ]
               then
                dir=`expr "$p" : "..\(.*\)"`
               else
                dir=`expr "$p" : "^..\(.*\)"`
              fi
              dirlibs="$dirlibs $dir"
            fi
	    ;;
        -l) 
           if [ "$expr_version" = 1.16 ]
            then
             lib=`expr "$p" : "..\(.*\)"`
            else
             lib=`expr "$p" : "^..\(.*\)"`
           fi
            found=0
            for l in $libsix4gl
            do 
              if [ $lib = $l ]; then
                found=1
                break
              fi
            done
            if [ $found = 0 ];then
              libs="$libs $lib"
            fi
	    ;;
        -*) 
	    ;;
	 *) case $next in 
	          I)
		    ;;
	          L)
                    optdirlibs="$optdirlibs -L $p"
                    dirlibs="$dirlibs $p"
		    ;;
	          link)
		    ;;
	          no)
		    if [ `basename $p` = "$filec" ];then
                      continue
                    fi
                    if [ "$expr_version" = 1.16 ]
                     then
	              ext=`expr "$p" : ".*\.\([^.]*\)$"`
                     else
	              ext=`expr "$p" : "^.*\.\([^.]*\)$"`
                    fi
                    if [ "$ext" = "a" -o "$ext" = "o" -o "$ext" = "so" ]; then
                      found=0
                      currl=`basename $p`
                      for l in $libsix4gl
                      do 
                        #echo_format "lib$l.$ext $currl"
                        if [ "lib$l.$ext" = $currl ];then
                          #echo "found"
                          found=1
                          break
                        fi
                        #echo " not found"
                      done
                      if [ $found = 0 ];then
                        libs="$libs $p"
                      fi
                    fi
		    ;;
            esac
            next="no"
	    ;;
  esac
 done

 syslibs=""
 inflibs=""
 nmlibs=""
 for l in $libs
 do
    if [ "$expr_version" = 1.16 ]
     then
      fc=`expr "$l" : "\(.\).*"`
     else
      fc=`expr "$l" : "^\(.\).*"`
    fi
    if [ $fc = "/" ]; then
      if [ "$expr_version" = 1.16 ]
       then
        a=`expr "$l" : "\($informixdir\)..*$"`
       else
        a=`expr "$l" : "^\($informixdir\)..*$"`
      fi
      if [ -n "$a" -a "$a" != 0 -a "$a" != 1 -a "$a" != 2 ]; then
        #a=`expr "$l" : "^$informixdir\(..*\)$"`
        nmlibs="$nmlibs $l"
        inflibs="$inflibs $l"
      else
        syslibs="$syslibs $l"
      fi
    else
      found=0
      for dir in $dirlibs
      do
        if [ "$expr_version" = 1.16 ]
         then
          a=`expr "$dir" : "\($informixdir\)..*$"`
         else
          a=`expr "$dir" : "^\($informixdir\)..*$"`
        fi
        for name in $l lib$l.a $l.o lib$l.so
        do
          if [ -r $dir/$name ];then
            found=1
            if [ -n "$a" -a "$a" != 0 -a "$a" != 1 -a "$a" != 2 ]; then
              nmlibs="$nmlibs $dir/$name"
              if [ $name = $l ];then
                inflibs="$inflibs $name"
              else
                inflibs="$inflibs -l$l"
              fi
            else
              syslibs="$syslibs $l"
            fi
          fi
        done
      done
      if [ $found = 0 ];then
        if [ "$expr_version" = 1.16 ]
         then
	  ext=`expr "$l" : ".*\.\([^.]*\)$"`
         else
	  ext=`expr "$l" : "^.*\.\([^.]*\)$"`
        fi
        if [ "$ext" != "a" -a "$ext" != "o" -a "$ext" != "so" ]; then
          syslibs="$syslibs -l$l"
        else
          syslibs="$syslibs $l"
        fi
      fi
    fi
 done
 #echo "inflibs $inflibs"
 #echo "syslibs $syslibs"
 #echo "nmlibs $nmlibs"
 if [ -n "$nmlibs" ];then
   echo "Ok"
 else
   echo "No"
 fi
}

#############################################################################
# giveMajorMinorVersion()
#
# DESCRIPTION : Try to search version of Informix
#
# DATE        : 07/12/98
##############################################################################
giveMajorMinorVersion () {
 version_product=$1
 shift
 release_product=$1
 shift 
 release_version_product=$1

 if [ -z "$version_product" ]
  then
   version_product="00"
 fi

 if [ -z "$release_product" ]
  then
   release_product="00"
 fi

 if [ -z "$release_version_product" ]
  then
   release_version_product="00"
 fi

 if [ `echo ${begech} "$version_product" ${endech}|wc -c` -lt 2 ]
  then
   gbl_major_version="0$version_product"
  else
   gbl_major_version="$version_product"
 fi  

 if [ `echo ${begech} "$release_product" ${endech}|wc -c` -lt 2 ]
  then
   gbl_minor_version="$release_product""0"
  else
   gbl_minor_version="$release_product"
 fi   
 
}

#############################################################################
# giveVersionNumber()
#
# DESCRIPTION : Try to search version of Informix
#
# DATE        : 07/12/98
##############################################################################
giveVersionNumber () {
 while [ $# != 0 ]
  do  
   shift 
   case "$1" in
    Version|VERSION|version)
      shift
      informix_version_number=`echo $1 | sed -e 's/\./ /g'`
      break
      ;;
    *)
      ;;
   esac
 done
 giveMajorMinorVersion $informix_version_number

}

#############################################################################
# search_version()
#
# DESCRIPTION : Try to search version of Informix
#
# DATE        : 03/09/97
##############################################################################

search_version(){

 echo_format "Search Informix version compatibility"
# allProgs="esql dbaccess c4gl"

 allProgs="esql dbaccess c4gl"
 for progs in $allProgs
  do
   if [ -f $progs ]
    then
     case "$progs" in
      esql)
       test_vers=`esql -V | grep ESQL`
       ;;
      c4gl)
       test_vers=`c4gl -V | grep INFORMIX-4GL`
       ;;
      dbaccess)
       test_vers=`dbaccess -V | grep DB-Access`
       ;;
     esac
     
     giveVersionNumber $test_vers

     if [ "$gbl_major_version" -ge 06 ]
      then
       if [ "$gbl_major_version" -gt 07 ]
        then
         echo "expect 9.xx"
         testDBS="ix914"
         found_ifx_version="1"
        else
         if [ "$gbl_minor_version" -gt 30 ]
          then
           echo "expect 9.xx"
           testDBS="ix914 ix711 ix410 ix501"
           found_ifx_version="1"
          else
           echo "expect 6.xx/7.xx"
           testDBS="ix711 ix914 ix410 ix501"
           found_ifx_version="1"
         fi
       fi
      else
       echo "expect 4.xx or 5.xx"
       if [ "$gbl_major_version" -eq 05 ]
        then
         found_ifx_version="1"
         testDBS="ix410 ix501 ix711 ix914"
        else
         found_ifx_version="1"
         testDBS="ix410 ix501 ix711 ix914"
       fi 
     fi
   fi
  
  if [ "$found_ifx_version" = "1" ]
   then
    break
  fi
 done
}

display_reslink() {

  echo "No"
  echo "----------"
  echo "with FGLLIBSQL=$FGLLIBSQL"
  echo "     FGLLIBSYS=$FGLLIBSYS"
  echo "     FGLCC    =$FGLCC"
  echo "     FGLDBS   =$FGLDBS"
  echo "     FGLCFLAGS=$FGLCFLAGS"
  echo "  --->"
  cat link.res 2>/dev/null
  echo "----------"
}

#############################################################################
# try_sql()
#
# DESCRIPTION : try to run a 4gl program with SQL
#
# DATE        : 03/09/97
##############################################################################

try_sql() {

  echo_format "Try a 4GL program with only SQL statements"
  cp $FGLDIR/lib/testbase.42m $tmpdir 2>/dev/null
  if [ $? != 0 ];then
    echo "No"
    echo " Problem to copy file '$FGLDIR/lib/testbase.42m'"
    return
  fi
  cd $tmpdir
  ./link4js testbase 2>sql.res 1>&2
  if [ $? != 0 ];then
    echo "No"
    echo_nonl " -> "
    cat sql.res|grep -v "^Demo"
    return
  fi
  echo "Ok"
 
}

#############################################################################
# try_link_runner()
#
# DESCRIPTION : 1 try to link the Four J's runner
#
# DATE        : 03/09/97
##############################################################################

try_link_runner() {

  FGLLIBSQL="$optdirlibs $inflibs"
  export FGLLIBSQL
  FGLLIBSYS="$slib"
  export FGLLIBSYS
  FGLCC="$FCC"
  export FGLCC
  FGLDBS=$dbs
  export FGLDBS
  FGLCFLAGS="$ANSICFLAGS $opt"
  export FGLCFLAGS
  #echo "FGLLIBSQL=$FGLLIBSQL"
  #echo "FGLLIBSYS=$FGLLIBSYS"
  #echo "FGLDBS=$FGLDBS"

  nbtry=`expr $nbtry + 1`
  if [ "$1" != 0 ];then
    echo_format "Try nr $nbtry $1"
  fi

  cd $tmpdir
  link=0
  fglmkrun -o link4js  2>link.res 1>&2
  if [ $? = 0 ]; then
    echo "Ok"
    link=1
    return
  fi
  #display_reslink
  grep -v "^Build" link.res >link.ver
  get_undef_fct link4js  link.ver
  if [ $minlfct -gt $nblfct ];then 
    minlfct=$nblfct
    okinflibs="$inflibs"
    oksyslibs="$slib"
    okcflags="$ANSICFLAGS $opt"
    okdbs="$dbs"
    okcc="$FCC"
  fi
  echo "No"
 
}
#############################################################################
# link_runner_withsys()
#
# DESCRIPTION : try to link with one sys libs 
#
# DATE        : 03/09/97
##############################################################################

link_runner_withsys() {
  onlyone="$1"
  while read lsys
  do
    if [ -z "$lsys" ];then
      continue
    fi
    nopt=0
    for opt in "$tryopt" "tryopt2"
    do
      nopt=`expr $nopt + 1`
      if [ $nopt = 2 -a "$tryopt" = "$tryopt2" ];then
        break
      fi
      slib=""
      for l in $syslibs
      do
        found=0
        for l4 in $lsys
        do
          if [ $l = $l4 ];then
            found=1
            break
          fi
        done
        if [ $found = 0 ];then
          slib="$slib $l"
        fi
      done
      slib="$slib $lsys"
      try_link_runner "to link the runner"
      if [ -n "$knownadd" ];then
        slib="$slib $knownadd"
        try_link_runner "to link the runner"
      fi

      if [ $link = 1 ];then
        try_sql
        if [ $try711 = 0 -a "$dbs" != "ix914" -a "$dbs" != "ixgen" ];then
          olddbs=$dbs
          dbs="ix711"
          echo_format "Trying to link runner with frontend 7.xx"
          try_link_runner 0
          if [ $link = 0 ];then
            link=1
            FGLDBS=$olddbs
          else
            try_sql
          fi
        fi
        break
      else
        if [ "$onlyone" = 1 ];then
          break 2
        fi
      fi
    done
    if [ $link = 1 ];then
      break
    fi
  done
}

#############################################################################
# link_runner()
#
# DESCRIPTION : Try to link the Four J's runner
#
# DATE        : 03/09/97
##############################################################################

link_runner(){
 if [  -z "$nmlibs" ];then
   return
 fi
 onlyone="$1"
 found=0
 try711=0
 cd $tmpdir
 for dbs in $testDBS
 do
   if [ $dbs = "ix711" ];then
     try711=1
   fi
   link_runner_withsys "$onlyone" <testlib.dat
   if [ $link = 1 -o "$onlyone" = 1 ];then
     break
   fi
 done
 if [ $link = 1 ];then
   okinflibs="$FGLLIBSQL"
   oksyslibs="$FGLLIBSYS"
   okcflags="$FGLCFLAGS"
   okdbs="$FGLDBS"
   okcc="$FCC"
   libs=""
   tlibs=""
   foundtestlib=0
   for l in $FGLLIBSYS
   do
     found=0
     for lt in $libstest
     do
       if [ "-l$lt" = "$l" ]; then
         found=1
         break
       fi
     done
     if [ $found = 1 ];then
       foundtestlib=1
       if [ $lt = "x" -o $lt = "bsd" ];then
         tlibs="$l $tlibs"
       else
         tlibs="$tlibs $l"
       fi
     else
       libs="$libs $l"
     fi
   done
   if [ $foundtestlib = 1 ];then
     reduce=0
     dbs="$FGLDBS"
     slib="$libs"
     nbtry=0
     #echo "oksyslibs $oksyslibs FGLLIBSYS $FGLLIBSYS libs $libs tlibs $tlibs"
     for i in "" $tlibs
     do
       slib="$slib $i"
       try_link_runner "to reduce system libraries"
       if [ $link = 1 ];then
         reduce=1
         break
       fi
     done
     if [ $reduce = 1 ];then
       oksyslibs="$FGLLIBSYS"
     else
       link=1
     fi
   fi
 fi
}

#############################################################################
# analyze_esql_version()
#
# INPUT       : $1 = Value send by the find_esql_version function
#
#
# DESCRIPTION : Return TRUE if ESQL version is > to 7.3 and FALSE for
#               other version
#               This is a public function.
#
# DATE        : 28/09/98
# UPDATE      : 28/09/98
##############################################################################

analyze_esql_version () {
informix_dbs_version=$1
shift
informix_dbs_release=$1
shift
informix_dbs_release_version=$1

 if [ `echo ${begech} "$informix_dbs_version" ${endech} |wc -c` -lt 2 ]
  then
   informix_dbs_version="0$informix_dbs_version"
 fi  

 if [ `echo ${begech} "$informix_dbs_release" ${endech}|wc -c` -lt 2 ]
  then
   informix_dbs_release="$informix_dbs_release""0"
 fi   
 
  if [ "$informix_dbs_version" -ge 07 ]
   then
    if [ "$informix_dbs_version" -gt 07 ]
     then
      echo "TRUE"
     else
      if [ $informix_dbs_release -gt 30 ]
       then
        echo "TRUE"
       else
        echo "FALSE"
      fi
    fi
   else
    echo "FALSE"
  fi
}

#############################################################################
# find_esql_version()
#
# INPUT       : $1 = Value of esql -V
#
#
# DESCRIPTION : Return all if ESQL version is > to 7.3
#               This is a public function.
#
# DATE        : 28/09/98
# UPDATE      : 28/09/98 
##############################################################################

find_esql_version () {

 while [ $# != 0 ]
  do  
   shift 
   case "$1" in
    Version)
      shift
      informix_version=`echo $1 | sed -e 's/\./ /g'`
      break
      ;;
    *)
      ;;
   esac
 done

echo `analyze_esql_version $informix_version`
}


#############################################################################
# get_informix_libs()
#
# INPUT       : $1 = Informix directory ($INFORMIXDIR)
#
#
# DESCRIPTION : Return all needed Informix libraries.
#               This is a public function.
#
# DATE        : 20/11/96
# UPDATE      : 03/09/97
##############################################################################

get_informix_libs() {
 informixdir=$1

echo_format "Informix directory found ?"
if [ -r $informixdir ];then
  echo "Ok"
else
  echo "No"
  echo "Exit program."
  exitp 1
fi
foundscript=0

minlfct=999
nbtry=0

for f in $shi
do
  obj_files=""
  informix_730="FALSE"

  echo_format "Informix '$f' found ?"
  if [ -f "$informixdir/bin/$f" ]; then
     echo "Ok"
     infcc=$f

     if [ $f != "cfglgo" ]; then

      echo_format "$f is compliant Informix 9.14 or higher"
      if [ $f = "c4gl" ]; then
        esql_vers=`c4gl -V | grep INFORMIX-4GL`
       else
        esql_vers=`esql -V | grep ESQL`
      fi

      if [ ! -z "$esql_vers" ]; then
       informix_730=`find_esql_version $esql_vers`
      fi

      if [ "$informix_730" = TRUE ]; then
        echo "Yes"
        flag_to_test="TRUE" 
       else
        echo "No"
        flag_to_test="TRUE"
      fi
      else
       flag_to_test="TRUE"
     fi

     if [ "$flag_to_test" = TRUE ]
      then
       if [ $f = "c4gl" ]; then
        file=te${f}.4gl
       else
        file="te${f}.c"
       fi
       filec="te${f}.c"
       source=$file
       foundscript=1
       foundscript_name="$f" # Modif Es
       link_with="$f"
       echo_format "Linking the runner with $f"
       cd $tmpdir
       fglmkrun -sh $f -o link4js 2>link.res 1>&2
       if [ $? != 0 ]
        then
         echo "No"
        else
         echo "Ok"
         FGLSHELL="$f"
         if [ "$max" != 1 ]
          then
           SHELL_COMMENT=""
           link=1
           #break
         fi
       fi

     get_compile_line $f
     if [ $foundlink = 1 ];then
       extract_informix_libs $line
       search_version
       link_runner
       if [ $link = 1 ];then
         break
       fi
     fi
 
     fi
  else # Modif Es
     echo "No"  # Modif Es
  fi # Modif Es
done # Modif Es

if [ $foundscript = 0 ];then
  echo_format "Informix script for link"
  echo "Not found"
  if [ $max = 0 ];then
    echo 
    echo "Please run '$progname -max' to make more tries."
    echo 
    echo "Exit program."
    exitp 1
  fi
fi
}

#############################################################################
# search_order()
#
# DESCRIPTION : try to found order of directory
#
# DATE        : 20/11/96
##############################################################################

search_order(){
echo_format "Search order of $text libraries"
echo "Running"
echo_nonl "Please wait..."
slib="$inflibs"
i=0
cd $tmpdir
cat fct/* |sort -u >allfct
foundord=1
#echo "==============="
#echo "inflibs $inflibs"
#echo "==============="
while [ $foundord = 1 ]
do
  FGLLIBSQL="$slib"
  export FGLLIBSQL
  #echo "FGLLIBSQL=$slib"
  fglmkrun -o link4js $slib >badlink 2>&1
  if [ $? != 0 ];then
    grep -v "^Build" badlink >link.ver
    get_undef_fct link4js  link.ver
    echo $lfct|sed 's/ /\
/g' >lfct
    #cat lfct
    a=`cat allfct lfct|sort|uniq -d|wc -l`
  #  echo "a is $a"
    if [ $a = 0 ];then
      inflibs="$slib"
      #echo "Not other."
      break
    fi
  else
    break
  fi
  echo_nonl "."
  i=0
  foundord=0
  #echo "found nblfct =$nblfct"
  for l in $inflibs
  do
    b=`basename $l`
    echo "Try $l"
    a=`cat fct/$b lfct|sort|uniq -d|wc -l`
    #echo "nb is $a"
    if [ $a != 0 ];then
      #echo "add $l"
      echo_nonl "."
      foundord=1
      slib="$slib $l"
    fi
  done
done
if [ $foundord = 0 ];then
  echo "No"
  echo "Cannot find link with existing Informix libraries"
  return
fi
#echo "+---------"
#echo "$inflibs"
#echo "+---------"
echo "Ok"
}


#############################################################################
# phys_inflib()
#
# DESCRIPTION : try to find all Informix libraries to make a runner
#
# DATE        : 20/11/96
##############################################################################

phys_inflib() {

 if [ $foundnm = 0 ];then
   echo "No"
   echo "No 'nm' command found before"
   exitp 1
 fi
 if [ ! -r $INFORMIXDIR/lib ];then
   echo "No"
   echo "Directory '\$INFORMIXDIR/lib' doesn't exist"
   exitp 1
 fi
 if [ $? != 0 ];then
   echo "No"
   echo "Cannot access directory '\$INFORMIXDIR/lib'"
   exitp 1
 fi
 libsix4gl="ncolor fmain fglgo nforms 4gl 4io nmenu rdsterm fe db4io dbgo \
            fgldb sace sperf"
 nbe=0
 minlfct=999
 nbtry=0
if [ $typlib = "-static" ];then
  typlib1="a"
  typlib2="so"
else
  typlib1="so"
  typlib2="a"
fi
for onlyone in 1 0
do
 testlink=0
 if [ $onlyone = 0 ]; then
   if [ $nbtry -gt 5 ];then
     echo "******************************************************************************"
     echo "* WARNING! This program can take a quite long time before you get a result!  *"
     echo "******************************************************************************"
   fi
   create_envf4gl 0
   create_envf4gl 0 csh
 fi
 for cext in $typlib1 $typlib2
 do
   first=1
   otherdir=1
   fdir=""
   while [ $otherdir = 1 ]
   do
     if [ $cext = "a" ];then 
       text="static"
     else
       text="shared"
     fi
     if [ $first = 1 ];then
       echo_format "Searching existing Informix $text libraries"
     else
       echo_format "Searching other existing Informix $text libraries"
     fi
     if [ $typlib = "-static" ];then
       nolibtype="shr"
       gdlibtype=""
     else
       nolibtype=""
       gdlibtype="shr"
     fi
     otherdir=0
     lfile=""
     cd $INFORMIXDIR/lib
     for f in *
     do
       if [ -d $f ];then
         if [ $otherdir = 0 ];then
           for d in $fdir
           do
              if [ $d = $f ];then
                continue 2
              fi
           done
           otherdir=1
           fdir="$fdir $f"
           lfile="$lfile `echo $f/*`"
         fi
       else
         lfile="$lfile $f"
       fi
     done
     if [ $first = 1 -o $otherdir = 1 ];then
       echo "Ok"
       clib=""
       nbclib=0
       #echo "lfile -> $lfile"
       for f in $lfile
       do
         if [ "$expr_version" = 1.16 ]
          then
           ext=`expr "$f" : ".*\.\([^.]*\)$"`
          else
           ext=`expr "$f" : "^.*\.\([^.]*\)$"`
         fi
         if [ "$ext" = "o" -o "$ext" = $cext ];then
           suf=`basename $f`
           found=0
           for l in $libsix4gl
           do
              if [ lib$l.$cext = $suf ];then
                found=1
                break
              fi
           done
           if [ $found = 0 ];then
             clib="$clib $f"
             nbclib=`expr $nbclib + 1`
           fi
         fi
       done
       cd $tmpdir
       nbe=`expr $nbe + 1`
       echo_format "Extracting function names of $text libraries ($nbe)"
       echo "Running"
       echo_nonl "Please wait..."
       i=0
       mkdir nb 2>/dev/null
       mkdir fct 2>/dev/null
       for l in $clib
       do
         f=`basename $l`
         eval "nm $INFORMIXDIR/lib/$l 2>/dev/null $nmcmd " >fct/$f
         cat fct/$f|wc -l >nb/$f
         echo_nonl "."
       done
       echo ".Ok"
       #echo "----------"
       #echo "clib($nbclib ,$nolibtype,$gdlibtype) $clib"
       #echo "----------------"
       if [ -z "$clib" ];then
         first=0
         continue
       fi
       if [ $nbclib -gt 1 ];then 
         wshr=1
         for libext in "$nolibtype" t ix "[i|t]x*" "$gdlibtype"
         do
           echo_format "Suppressing duplicate $text libraries ($nbe)"
           echo "Running"
           echo_nonl "Please wait..."
           case "$libext" in
		t)  suflib=ix
			;;
		ix) suflib=t
			;;
		*)  suflib=""
			;;
           esac
           ftlibs=""
           extlibs=""
           reduce=1
           if [ -n "$libext" ];then
             for l in $clib
             do
               f=`basename $l`
               if [ $libext = "shr" ];then
                 a=`expr "$f" : "lib..*\($libext\)\.[^\.]*"`
                 if [ -n "$a" -a "$a" != 0 -a "$a" != 1 -a "$a" != 2 ]; then
                   wshr=1
                 fi
               else
                 a=`expr "$f" : "lib\($libext\)..*"`
               fi
               if [ -z "$a" ];then
                 ftlibs="$ftlibs $l"
                 if [ -n "$suflib" ];then
                   a=`expr "$f" : "lib\($suflib\)..*"`
                   if [ -n "$a" -a "$a" != 0 -a "$a" != 1 -a "$a" != 2 ]; then
                     a=`expr "$f" : "lib$suflib\([^\.]*\)\.[^\.]*"`
                     extlibs="$extlibs $a"
                   fi
                 fi
               fi
             done
           else
             if [ "$typlib" = "-shared" -a $cext = "a" ];then
               # keep "shr" extensions
               found=0
               for l in $clib
               do
                 f=`basename $l`
                 a=`expr "$f" : "lib\(..*\)shr\.[^\.]*"`
                 if [ -n "$a" -a "$a" != 0 -a "$a" != 1 -a "$a" != 2 ]; then
                   ftlibs="$ftlibs $l"
                   reduce=0
                   found=1
                 fi
               done
               if [ $found = 0 ];then
                 ftlibs="$clib"
               fi
             else
               ftlibs="$clib"
             fi
           fi
           if [ -n "$extlibs" ];then
             nlibs=""
             for l in $ftlibs
             do
               found=0
               f=`basename $l`
               if [ "$expr_version" = 1.16 ]
                then
                 a=`expr "$f" : "lib\($suflib\)..*"`
                else
                 a=`expr "$f" : "^lib\($suflib\)..*"`
               fi
               if [ -z "$a" ];then
                if [ "$expr_version" = 1.16 ]
                 then
                  a=`expr "$f" : "lib\([^\.]*\)\.[^\.]*"`
                 else
                  a=`expr "$f" : "^lib\([^\.]*\)\.[^\.]*"`
                fi
                 if [ -n "$a" -a "$a" != 0 -a "$a" != 1 -a "$a" != 2 ]; then
                   for n in $extlibs
                   do
                     if [ $a = $n ];then
                       found=1
                       break
                     fi
                   done
                 fi
               fi
               if [ $found = 0 ];then 
                 nlibs="$nlibs $l"
               fi
             done
             ftlibs="$nlibs"
           fi
           if [ -z "$ftlibs"  ];then
             echo ".Ok"
             continue
           else
             if [ "$libext" = "shr" -a $wshr = 0 ];then
               echo ".Ok"
               continue
             fi
           fi
           #continue
           badlibs=""
           inflibs=""
           i=0
           nbl=0
           #echo "----------"
           #echo "| ftlibs: $ftlibs"
           #echo "----------"
           for f1 in $ftlibs
           do
             b1=`basename $f1`
             echo_nonl "."
             i=`expr $i + 1`
             j=0
             if [ $reduce = 1 ];then
               okl=0
               for f2 in $ftlibs
               do
                 b2=`basename $f2`
                 j=`expr $j + 1`
                 if [ $i -ge $j ];then
                   continue
                 fi
                 for l in $badlibs
                 do
                   if [ $f2 = $l ];then
                     continue 2
                   fi
                 done
                 okl=1
                 a=`cat fct/$b1 fct/$b2|sort|uniq -d|wc -l`
                 if [ $a != 0 ];then
                   if [ `cat nb/$b1` -lt `cat nb/$b2` ];then
                     okl=0
                     break
                   fi
                 fi
               done
             else
               okl=1
             fi
             if [ $okl = 1 ];then 
               nbl=`expr $nbl + 1`
               inflibs="$inflibs $INFORMIXDIR/lib/$f1"
               #echo "$f1 is ok"
             else
               badlibs="$badlibs $f1"
               #echo "$f1 is a bad lib"
             fi
           done
           echo ".Ok"
           #echo "----------------"
           #echo "inflibs $inflibs"
           #echo "----------------"
           if [ $depend = 1 -a $nbl -gt 1 ];then 
             search_order
           fi
           nmlibs="$inflibs"
           search_version
           testlink=1
           link_runner $onlyone
           if [ $link = 1 ];then
             break 4
           fi
         done
       else
         for i in $clib
         do
           inflibs="$inflibs $INFORMIXDIR/lib/$i"
         done
         nmlibs="$inflibs"
         search_version
         testlink=1
         link_runner $onlyone
         if [ $link = 1 ];then
           break 3
         fi
       fi
     else
       echo "No"
     fi
   done
   first=0
 done
done
 if [ $testlink = 0 ];then
   echo "No"
   echo "No Informix libraries found."
   exitp 1
 fi
}

#############################################################################
# create_envf4gl()
#
# DESCRIPTION : Create environment file
#               This is a public function.
#
# UPDATE      : 04/09/97
##############################################################################

create_envf4gl() {
 dname_libs=""
 befld_path=""
 item_ld_path=""
 share_lib_path=""

 we="$1"
 if [ "$2" != "" ];then
   nameEnvf4gl=envf4gl.$2
   beg="setenv "
   sep=" "
   shell="csh"
 else
   nameEnvf4gl=envf4gl
   beg=""
   sep="="
   shell="sh"
 fi
 if [ "$we" != 0 ];then
   echo_format "Creation of file '$nameEnvf4gl"
 fi
 cd $source_dir

 echo "#!/bin/$shell" >$nameEnvf4gl
 echo "" >>$nameEnvf4gl
 echo "#------- Set variables for compilation" >>$nameEnvf4gl
 echo "" >>$nameEnvf4gl
 echo "${beg}SHELL${sep}/bin/$shell" >>$nameEnvf4gl
 if [ "$shell" = "sh" ];then echo "export SHELL" >>$nameEnvf4gl;fi
 echo "" >>$nameEnvf4gl
 echo "#------- Install directory" >>$nameEnvf4gl
 echo "" >>$nameEnvf4gl
 echo "${beg}FGLDIR${sep}$FGLDIR" >>$nameEnvf4gl
 if [ "$shell" = "sh" ];then echo "export FGLDIR" >>$nameEnvf4gl;fi
 
 echo "" >>$nameEnvf4gl
 echo "#------- Informix directory" >>$nameEnvf4gl
 echo "" >>$nameEnvf4gl
 echo "${beg}INFORMIXDIR${sep}$INFORMIXDIR" >>$nameEnvf4gl
 if [ "$shell" = "sh" ];then echo "export INFORMIXDIR" >>$nameEnvf4gl;fi

 echo "" >>$nameEnvf4gl
 echo "#------- Upgrade the path" >>$nameEnvf4gl
 echo "" >>$nameEnvf4gl
 echo "${beg}PATH${sep}\"\$FGLDIR/bin:\$INFORMIXDIR/bin:\$PATH\"" >>$nameEnvf4gl
 if [ "$shell" = "sh" ];then echo "export PATH" >>$nameEnvf4gl;fi

 echo "" >>$nameEnvf4gl
 echo "#------- Database frontend version" >>$nameEnvf4gl
 echo "" >>$nameEnvf4gl
 echo "${beg}FGLDBS${sep}\"$okdbs\"" >>$nameEnvf4gl
 if [ "$shell" = "sh" ];then echo "export FGLDBS" >>$nameEnvf4gl;fi

 if [ "$okcc" != "cc" ];then
   echo "" >>$nameEnvf4gl
   echo "#------- C Compiler " >>$nameEnvf4gl
   echo "" >>$nameEnvf4gl
   echo "${beg}FGLCC${sep}\"$okcc\"" >>$nameEnvf4gl
   if [ "$shell" = "sh" ];then echo "export FGLCC" >>$nameEnvf4gl;fi
 fi
 a=`echo $okcflags|sed 's/ //g'`

 if [ -n "$a" ];then
   echo "" >>$nameEnvf4gl
   echo "#------- Flags for C compiler" >>$nameEnvf4gl
   echo "" >>$nameEnvf4gl
   echo "${beg}FGLCFLAGS${sep}\"$okcflags\"" >>$nameEnvf4gl
   if [ "$shell" = "sh" ];then echo "export FGLCFLAGS" >>$nameEnvf4gl;fi
 fi

 if [ "$FGLSHELL" ]
  then
   echo "" >>$nameEnvf4gl
   echo "#------- SHELL Compiler parameter " >>$nameEnvf4gl
   echo "" >>$nameEnvf4gl
   echo "${beg}FGLSHELL${sep}\"$FGLSHELL\"" >>$nameEnvf4gl
   if [ "$shell" = "sh" ];then echo "export FGLSHELL" >>$nameEnvf4gl;fi
 fi
 echo "" >>$nameEnvf4gl
 echo "#------- Informix SQL libraries" >>$nameEnvf4gl
 echo "" >>$nameEnvf4gl
 echo_nonl "${SHELL_COMMENT}${beg}FGLLIBSQL${sep}\"" >>$nameEnvf4gl


 nb=0
 for l in $okinflibs
 do
   nb=`expr $nb + 1`
 done
 i=0
 nextL=0
 Ldir=""
 firstwrite=1
 oldl=""
 finished=0
 for l in $okinflibs
 do
   i=`expr $i + 1`
   if [ "$l" = "-L" ];then
     nextL=1
     continue
   fi
   if [ $nextL = 1 ];then
     nextL=0
     for Ll in $Ldir
     do
       if [ $l = $Ll ];then
         continue 2
       fi
     done
     Ldir="$Ldir $l"
     opt="-L "
   else
     opt=""
   fi
   if [ "$oldl" = $l ];then
     continue
   fi
   if [ $firstwrite = 0 ];then
     echo_nonl "${SHELL_COMMENT}           " >>$nameEnvf4gl
   fi
   if [ "$expr_version" = 1.16 ]
    then
     a=`expr "$l" : "\($INFORMIXDIR\)..*$"`
    else
     a=`expr "$l" : "^\($INFORMIXDIR\)..*$"`
   fi
   if [ -n "$a" -a "$a" != 0 -a "$a" != 1 -a "$a" != 2 ]; then
    if [ "$expr_version" = 1.16 ]
     then
      a=`expr "$l" : "$INFORMIXDIR\(..*\)$"`
     else
      a=`expr "$l" : "^$INFORMIXDIR\(..*\)$"`
    fi
    if [ -n "$a" -a "$a" != 0 -a "$a" != 1 -a "$a" != 2 ]; then
     echo_nonl "$opt\$INFORMIXDIR$a" >>$nameEnvf4gl
    fi
   else
    if [ "$expr_version" = 1.16 ]
     then
      a=`expr "$l" : "-L\($INFORMIXDIR\)..*$"`
     else
      a=`expr "$l" : "^-L\($INFORMIXDIR\)..*$"`
    fi
     if [ -n "$a" -a "$a" != 0 -a "$a" != 1 -a "$a" != 2 ]; then
      if [ "$expr_version" = 1.16 ]
       then
        a=`expr "$l" : "-L$INFORMIXDIR\(..*\)$"`
       else
        a=`expr "$l" : "^-L$INFORMIXDIR\(..*\)$"`
      fi
      if [ -n "$a" -a "$a" != 0 -a "$a" != 1 -a "$a" != 2 ]; then
       echo_nonl "-L\$INFORMIXDIR$a" >>$nameEnvf4gl
      fi
     else
       echo_nonl "$opt$l" >>$nameEnvf4gl
     fi
   fi
    if [ -z "$a" -o "$a" = 0 -o "$a" = 1 -o "$a" = 2 ]; then
     a=""
    fi
    if [ $i != $nb ];then
     if [ -z "$a" ]; then
      if [ ! -z "$l" ]; then
       echo " \\" >>$nameEnvf4gl
      fi
      else
      echo " \\" >>$nameEnvf4gl
     fi
    else
     if [ -z "$a" ]; then
      if [ ! -z "$l" ]; then
       finished=1
       echo "\"" >>$nameEnvf4gl
      fi
      else
       finished=1
       echo "\"" >>$nameEnvf4gl
     fi
    fi
   firstwrite=0
   oldl=$l
 done
 if [ $finished = 0 ];then
   echo "\"" >>$nameEnvf4gl
 fi
 if [ "$shell" = "sh" ];then echo "${SHELL_COMMENT}export FGLLIBSQL" >>$nameEnvf4gl;fi

 for d in $okinflibs
  do
   firstokinflibs=`expr "$d" : "\(.\).*$"`
   secondokinflibs=`expr "$d" : ".\(.\).*$"`
   if [ "$firstokinflibs" != "-" -a "$secondokinflibs" != "l" ]
    then
     if [ "$firstokinflibs" != "-" -a "$secondokinflibs" != "L" ]
      then
       dname_libs="${dname_libs} `dirname $d`"
      else
       temp_d=`expr "$d" : "-L\(.*\)$"`
       if [ ! -z "$temp_d" ]
        then
         if [ -d "$temp_d" ]
          then
           dname_libs="${dname_libs} $temp_d"
           temp_d=""
          else
           dname_libs="${dname_libs} `dirname $temp_d`"
           temp_d=""
         fi
       fi
     fi
   fi
 done
 
 echo $dname_libs|sed 's/ /\
/g' >dname_libs$$
 befld_path=`cat dname_libs$$|sort|uniq`
 rm dname_libs$$ 1>/dev/null 2>/dev/null

 for item_ld_path in $befld_path
  do
   if [ -z "$share_lib_path" ]
    then
     share_lib_path="\$INFORMIXDIR/"`expr "$item_ld_path" : "$INFORMIXDIR/\(.*\)$"`
    else
     share_lib_path="${share_lib_path}:\$INFORMIXDIR/"`expr "$item_ld_path" : "$INFORMIXDIR/\(.*\)$"`
   fi
 done 

 if [ ! -z "$share_lib_path" ]
  then
   echo "" >>$nameEnvf4gl
   echo "#------- LD_PATH_DIRECTORY " >>$nameEnvf4gl
   echo "" >>$nameEnvf4gl
   if [ "$shell" = "sh" ];then
     echo "${beg}LD_LIBRARY_PATH${sep}\"$share_lib_path:\${LD_LIBRARY_PATH=}\"" >> $nameEnvf4gl
     echo "export LD_LIBRARY_PATH" >>$nameEnvf4gl
   else
     echo "if ( \${?LD_LIBRARY_PATH} ) then" >>$nameEnvf4gl
     echo "  setenv LD_LIBRARY_PATH \"$share_lib_path:\$LD_LIBRARY_PATH\"" >>$nameEnvf4gl
     echo "else" >>$nameEnvf4gl
     echo "  setenv LD_LIBRARY_PATH \"$share_lib_path\"" >>$nameEnvf4gl
     echo "endif" >>$nameEnvf4gl
   fi
 fi

 echo "" >>$nameEnvf4gl
 echo "#------- System libraries" >>$nameEnvf4gl
 echo "" >>$nameEnvf4gl
 echo_nonl "${SHELL_COMMENT}${beg}FGLLIBSYS${sep}\"" >>$nameEnvf4gl
 nb=0
 for l in $oksyslibs
 do
   nb=`expr $nb + 1`
 done
 i=0
 finished=0
 for l in $oksyslibs
 do
   i=`expr $i + 1`
   if [ $i != 1 ];then
     echo_nonl "${SHELL_COMMENT}           " >>$nameEnvf4gl
   fi
   echo_nonl "$l" >>$nameEnvf4gl
   if [ $i != $nb ];then
     echo " \\" >>$nameEnvf4gl
   else
     finished=1
     echo "\"" >>$nameEnvf4gl
   fi
 done
 if [ $finished = 0 ];then
   echo "\"" >>$nameEnvf4gl
 fi
 if [ "$shell" = "sh" ];then echo "${SHELL_COMMENT}export FGLLIBSYS" >>$nameEnvf4gl;fi

 if [ "$we" != 0 ];then
   echo "Ok"
 fi
 have_env=1
}

##################################### MAIN ####################################
searchlib=1
max=0
shi="esql c4gl cfglgo"
next=""
nocls=""
typlib="-static"
testDBS="ix914 ixgen ix711 ix410 ix501"


for p in $*
do
  case $p in
     -inst) 
        from_install="y"
        nocls="y"
        ;;
     -nocls) 
        nocls="y"
        ;;
     -l) searchlib=1
	;;
     -o) next="o"
	;;
     -max) max=1
	;;
     -v|-V) echo "Searching libraries to link the runner Version 2.05.1b"
         exit 0
	;;
     -h|help)
         usage
	;;
     -sh) next="sh"
	;;
     -tl) next="tl"
	;;
     -sys) next="sys"
	;;
     *) case "$next" in
                sys) system_name="$p"
                ;;
		o) runner=$p
		;;
		sh) shi="$p"
		;;
		tl) case "$p" in
			shared) typlib="-shared"
				;;
			static) typlib="-static"
				;;
			-shared|-static) typlib="$p"
				;;
			*) echo "Bad type of libraries $p"
		   	   exit 0
		           ;;
                    esac
		;;
		*) echo "Bad parameters $p"
		   exit 0
		;;
        esac
  esac
done
if [ -z "$from_install" ]
 then
  from_install="n"
fi
if [ -z "$nocls" ]
 then
  nocls="n"
fi
if [ "$nocls" = n ]
 then
  clear 2>/dev/null
  if [ $? != 0 ]
   then
    tput clear 2>/dev/null
  fi
fi

a=`expr --version 2>/dev/null`
if [ $? = 0 -a ! -z "$a" ]
 then
  expr_version=`expr "$a" : ".* \(.*\)$"`
fi 
echo 'Please, wait ...'
if [ $? != 0 ]
then
  tput clear 2>/dev/null
fi
#----- test of environment
if [ -z "$INFORMIXDIR" ]
then
  echo 'Error: env. INFORMIXDIR variable is needed.'
  exitp 1
fi
if [ -z "$FGLDIR" ]
then
  echo 'Error: env. FGLDIR variable is needed.'
  exitp 1
fi

#----- directory /tmp must be present
if [ ! -d "/tmp" ]; then
  echo 'Error: Directory /tmp not found.'
  echo '$progname aborted. Exit.'
  exitp 1
fi

#----- Create directory for temporary files.
rm -f envf4gl.lib
if [ $? != 0 ]; then
  echo "Cannot erase file 'envf4gl.lib'"
  echo "Please, check your access rights or use the 'root' account to run this program"
  exitp 1
fi
if [ -d "$tmpdir" ]; then
  rm -f /tmp/err.buf
  rm -rf $tmpdir > /tmp/err 2> /tmp/err.buf
  if [ -f "tmp/err.buf" ]; then
   err=`cat tmp/err.buf|wc -l`
   if [  $err != 0 ]; then
     echo 'Unable to delete directory'$tmpdir'. Exit.'
     echo 'The following errors happened:'
     cat tmp/err.buf
     rm -rf /tmp/rtmpdir 2>/dev/null
     rm -f /tmp/*.buf 2>/dev/null
     exitp 1
   fi
  fi
fi
mkdir $tmpdir 2>/dev/null
if [ $? != 0 ];then
  echo "Cannot create directory '$tmpdir'"
  echo "Please, check your access rights or use the 'root' account to run this program"
  exitp 1
fi
chmod 0744  $tmpdir
if [ $? != 0 ];then
  echo "Cannot change mode of directory '$tmpdir'"
  echo "Please, check your access rights or use the 'root' account to run this program"
  echo "Remove that directory with command:'rm -r $tmpdir'"
  echo "And run this program again"
  exitp 1
fi
chmod go-wrx $tmpdir 2>/dev/null

#----- Identify host
detect_operating_system
if [ -z "$system_name" ]
 then
  echo_format "Identifying your operating system"
  echo "$os_name"
fi

#----- Source directory, search $progname
source_dir=`pwd`
if [ -r $source_dir/$progname ];then
  cp $source_dir/$progname $tmpdir
else
  type $progname >$tmpdir/path.fin
  if [ $? != 0 ];then
    echo "Cannot find location of $progname"
    echo "Please run this program in the same directory as '$progname'"
    exitp 1
  fi
  set `cat $tmpdir/path.fin`
  for i in $*
  do
    p=$i
  done
  if [ "$expr_version" = 1.16 ]
   then
    a=`expr "$p" : "\(.\).*"`
   else
    a=`expr "$p" : "^\(.\).*"`
  fi
  if [ $a = "/" ];then
    cp $p $tmpdir
  else
    cp $progname $tmpdir
  fi
  if [ $? != 0 ];then
    echo "Cannot copy file '$progname'"
    echo "Please run this program in the same directory as '$progname'"
    exitp 1
  fi
fi
cd $tmpdir
echo_format "Extracting test files"
self_extract $progname tec4gl.4gl
self_extract $progname teesql.c
self_extract $progname tecfglgo.c
self_extract $progname tenm.c
self_extract $progname natenm.c
if [ $max = 1 ];then
  self_extract $progname teli.c
  self_extract $progname teli1.c
  self_extract $progname teli2.c
  self_extract $progname unref.dat
  self_extract $progname syslib.dat
  echo "$sys4js" >tmp.dat
  echo "$sys4js2" >>tmp.dat
  cat tmp.dat syslib.dat >testlib.dat
else
  echo "$sys4js"  >testlib.dat
  echo "$sys4js2" >>testlib.dat
fi
echo "Ok"

#----- Check FGLDIR
if [ ! -r $FGLDIR/bin/fglmkrun ];then
  echo "Cannot find file '$FGLDIR/bin/fglmkrun'"
  echo "Please, check your environment"
  exitp 1
fi
PATH=$FGLDIR/bin:$PATH:$INFORMIXDIR/bin
export PATH

#----- Verify compiler
verify_compiler
if [ $max = 1 ];then
  search_nmcmd
fi

#----- Start Main program
link=0
get_informix_libs $INFORMIXDIR
if [ $foundscript = 0 -o $link = 0 ];then
  if [ $max = 1 ];then
    phys_inflib
  fi
fi
create_envf4gl 1
create_envf4gl 1 csh
if [ $link = 1 ];then
  if [ -n "$runner" ];then
    if [ $runner != "fglrun" ];then
      opt=" -o $runner"
      name=$runner
    else
      opt=""
      name=$FGLDIR/bin/fglrun
    fi
    echo_format "Link runner '$name'"
    . envf4gl
    fglmkrun $opt 2>$tmpdir/link.run 1>&2
    if [ $? = 0 ];then
      echo "Ok"
    else
      echo "No"
      echo "Following error happened:"
      cat $tmpdir/link.run
      exitp 1
    fi
    echo
    echo "Environment file 'envf4gl' or 'envf4gl.csh' sets variables."
    echo "Run the following commands to create a new runner:"
    echo ""
    echo "In Bourne Shell:"
    echo ". ./envf4gl"
    echo "fglmkrun$opt"
    echo ""
    echo "or in C Shell:"
    echo "source ./envf4gl.csh"
    echo "fglmkrun$opt"
  else
    echo
    echo "Searching libraries for the runner was successful."
    echo "Environment file 'envf4gl' or 'envf4gl.csh' sets variables."
    echo "Run the following commands to create your runner:"
    echo ""
    echo "In Bourne Shell:"
    echo ". ./envf4gl"
    if [ -n "$opt" ]
     then
       opt=" -add $opt"
    fi
    echo "fglmkrun$opt"
    echo ""
    echo "or in C Shell:"
    echo "source ./envf4gl.csh"
    echo "fglmkrun$opt"
  fi
  echo
  exitp 0
else
  echo "Link of the runner is unsuccessful."
  if [ $minlfct != 999 ];then
    echo "Minimum number of undefined link functions is: $minlfct"
    echo "Run the following commands to see exactly which one:"
    echo ". ./envf4gl"
    echo "fglmkrun"
  else
    echo "Check the documentation for more details."
  fi
  exitp 1
fi
exitp 0
######################
# Self extract files
######################
# File tec4gl.4gl
######################
@tec4gl.4gl@MAIN
@tec4gl.4gl@
@tec4gl.4gl@  DEFINE a CHAR
@tec4gl.4gl@
@tec4gl.4gl@  OPEN WINDOW w AT 2,2 WITH 20 ROWS, 65 COLUMNS
@tec4gl.4gl@
@tec4gl.4gl@  PROMPT "test " FOR CHAR a
@tec4gl.4gl@
@tec4gl.4gl@  CLOSE WINDOW w
@tec4gl.4gl@
@tec4gl.4gl@END MAIN
######################
# File teesql.c
######################
@teesql.c@
@teesql.c@main()
@teesql.c@{
@teesql.c@}
######################
# File tecfglgo.c
######################
@tecfglgo.c@#include "fgicfunc.h"
@tecfglgo.c@
@tecfglgo.c@cfunc_t usrcfuncs[] = 
@tecfglgo.c@    {
@tecfglgo.c@    0, 0, 0
@tecfglgo.c@    };
######################
# File tenm.c
######################
@tenm.c@int var1;
@tenm.c@static int var2=0;
@tenm.c@extern int var3;
@tenm.c@extern char *extfunct3(int);
@tenm.c@extern int extfunct4(int);
@tenm.c@extern long extfunct5(int);
@tenm.c@int funct1(int p1,int p2){
@tenm.c@return(0);
@tenm.c@}
@tenm.c@int funct2(int p1,int p2){
@tenm.c@return(0);
@tenm.c@}
@tenm.c@main(int argc,char **argv){
@tenm.c@extfunct3(0);
@tenm.c@extfunct4(0);
@tenm.c@extfunct5(0);
@tenm.c@}
######################
# File natenm.c
######################
@natenm.c@int var1;
@natenm.c@static int var2=0;
@natenm.c@extern int var3;
@natenm.c@extern char *extfunct3();
@natenm.c@extern int extfunct4();
@natenm.c@extern long extfunct5();
@natenm.c@int funct1(p1,p2)
@natenm.c@int p1;
@natenm.c@int p2;
@natenm.c@{
@natenm.c@return(0);
@natenm.c@}
@natenm.c@int funct2(p1,p2)
@natenm.c@int p1;
@natenm.c@int p2;
@natenm.c@{
@natenm.c@return(0);
@natenm.c@}
@natenm.c@main(argc,argv)
@natenm.c@int argc;
@natenm.c@char **argv;
@natenm.c@{
@natenm.c@extfunct3(0);
@natenm.c@extfunct4(0);
@natenm.c@extfunct5(0);
@natenm.c@}
######################
# File teli.c
######################
@teli.c@main(){
@teli.c@es2();
@teli.c@}
######################
# File teli1.c
######################
@teli1.c@es1(){
@teli1.c@}
######################
# File teli2.c
######################
@teli2.c@es2(){
@teli2.c@ es1();
@teli2.c@}
######################
# File unref.dat
######################
@unref.dat@__write $FGLDIR/lib/unref.c
@unref.dat@_write $FGLDIR/lib/unref.c
@unref.dat@fileno $FGLDIR/lib/unref1.c
@unref.dat@_fileno $FGLDIR/lib/unref1.c
@unref.dat@bigcrypt -lprot -ltinfo -lm -lx
######################
# File syslib.dat
######################
@syslib.dat@-lm
@syslib.dat@-lm -lx
@syslib.dat@-lm -lsocket
@syslib.dat@-lm -lsocket -lx
@syslib.dat@-lm -lsocket -lnsl
@syslib.dat@-lm -lsocket -lnsl -lx
@syslib.dat@-lm -lsocket -lnsl_s
@syslib.dat@-lm -lsocket -lnsl_s -lx
@syslib.dat@-lm -lsocket -lnsl_s -lproct_s -lx
@syslib.dat@-lm -lsocket -lnsl -lproct -lx
@syslib.dat@-lm -lsocket -lnsl -lmproct -lx
@syslib.dat@-lm -lsocket -lnsl_s -lproct_s 
@syslib.dat@-lm -lsocket -lnsl_s -lproct_s  -lx
@syslib.dat@-lm -lsocket -lnsl -lproct 
@syslib.dat@-lm -lsocket -lnsl -lmproct
@syslib.dat@-lnsl_s -lcrypt_i -lsocket -lm -lsuds -lx -lmalloc -lproc
@syslib.dat@-lnsl_s -lcrypt_i -lsocket -lm -lsuds -lx -lmalloc
@syslib.dat@-ltli -lc -lmsaa -lm
@syslib.dat@-lmproc -lext -losx -lelf -lm -lsocket -lnsl
@syslib.dat@-lpthreads -lmach -lm
@syslib.dat@-lpthreads -lm
